@@ -902,6 +902,34 @@ class DerivedDataApi(metaclass=Singleton):
                 print(f'failed to delete data <err_msg> {e} from {HedgeFundInvestorDivAndCarry.__tablename__}')
                 return False
 
+    def get_hedge_fund_custodian_data(self, manager_id: str, fof_id_list: Tuple[str] = ()):
+        with DerivedDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    HedgeFundCustodianData,
+                )
+                if fof_id_list:
+                    query = query.filter(
+                        HedgeFundCustodianData.fof_id.in_(fof_id_list),
+                    )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'failed to get_hedge_fund_custodian_data <err_msg> {e} from {HedgeFundCustodianData.__tablename__}')
+
+    def delete_hedge_fund_custodian_data(self, id_to_delete: int) -> bool:
+        with DerivedDatabaseConnector().managed_session() as session:
+            try:
+                session.query(
+                    HedgeFundCustodianData
+                ).filter(
+                    HedgeFundCustodianData.id == id_to_delete,
+                ).delete(synchronize_session=False)
+                session.commit()
+                return True
+            except Exception as e:
+                print(f'failed to delete data <err_msg> {e} from {HedgeFundCustodianData.__tablename__}')
+                return False
+
     def get_style_factor_ret(self, begin_date, end_date, time_para, index_list=['hs300']):
         try:
             start_date, end_date = RawDataApi().get_date_range(time_para, begin_date, end_date)
