@@ -2,6 +2,8 @@
 from typing import Dict, Any, Tuple, List
 import math
 import json
+import io
+import re
 
 import numpy as np
 import pandas as pd
@@ -10,7 +12,7 @@ from sqlalchemy.orm import sessionmaker
 
 from ...util.singleton import Singleton
 from ...constant import FOFTradeStatus, DividendCalcMethod
-from ..view.derived_models import HedgeFundInvestorPurAndRedemp, HedgeFundInvestorDivAndCarry
+from ..view.derived_models import HedgeFundInvestorPurAndRedemp, HedgeFundInvestorDivAndCarry, HedgeFundCustodianData
 from ..wrapper.mysql import DerivedDatabaseConnector
 from ..api.basic import BasicDataApi
 from ..api.derived import DerivedDataApi
@@ -440,6 +442,109 @@ class HedgeFundDataManager(metaclass=Singleton):
         df['event_type'] = FOFTradeStatus.DEDUCT_REWARD
         return df
 
+    def upload_custodian_data(self, manager_id: str, fof_id: str, datas: io.BytesIO):
+        '''
+        上传/更新私募基金托管数据
+
+        Parameters
+        ----------
+        manager_id : str
+            manager id
+        fof_id : str
+            fof id
+        datas : io.BytesIO
+            open 托管数据表格获得的 file object
+
+        Returns/Exceptions
+        -------
+        throw excepion if failed
+        '''
+
+        to_parse_date = pd.read_excel(datas, skiprows=3, nrows=1).columns.array[0]
+        the_date = re.findall(r'[0-9\-]+', to_parse_date)
+        assert the_date, f'parse date failed (info){to_parse_date}'
+        the_date = pd.to_datetime(the_date[0], infer_datetime_format=True).date()
+
+        df = pd.read_excel(datas, header=4, index_col=0)
+        equity = df.at['资产净值', '市值.1']
+        total_shares = df.at['实收资本', '数量']
+
+        mutual_funds_part1 = df.loc[(df.index > '1105.21.01') & (df.index < '1105.24'), ['科目名称', '数量', '市值.1', '行情']]
+        mutual_funds_part1 = mutual_funds_part1.reset_index(drop=True)
+        mutual_funds_part2 = df.loc[(df.index > '1105.24.01') & (df.index < '1108'), ['科目名称', '数量', '市值.1', '行情']]
+        mutual_funds_part2 = mutual_funds_part2.reset_index(drop=True)
+        mutual_funds = pd.concat([mutual_funds_part1, mutual_funds_part2], ignore_index=True).rename(columns={
+            '科目名称': 'fund_id',
+            '数量': 'shares',
+            '市值.1': 'mv',
+            '行情': 'nav',
+        })
+        mutual_funds['fund_id'] = mutual_funds.fund_id.map(lambda x: x + '!0')
+
+        hedge_funds = df.loc[(df.index > '1108.02.01') & (df.index < '1203'), ['数量', '市值.1', '行情']]
+        hedge_funds['fund_id'] = hedge_funds.index.map(lambda x: x.split()[0].split('.')[-1])
+        hedge_funds = hedge_funds.reset_index(drop=True).rename(columns={
+            '数量': 'shares',
+            '市值.1': 'mv',
+            '行情': 'nav',
+        })
+
+        deposit_in_bank = df.at['1002', '市值.1']
+        interest_receivable = df.at['1204', '市值.1']
+        try:
+            pur_amt_receivable = df.at['1207', '市值.1']
+        except KeyError:
+            pur_amt_receivable = np.nan
+        try:
+            sec_clearing_fund = df.at['3003', '市值.1']
+        except KeyError:
+            sec_clearing_fund = np.nan
+        try:
+            dividends_receivable = df.at['1203', '市值.1']
+        except KeyError:
+            dividends_receivable = np.nan
+        other_assets = interest_receivable + deposit_in_bank
+        payable_to_managers = df.at['2206', '市值.1']
+        payable_to_custodian = df.at['2207', '市值.1']
+        payable_to_admin_service = df.at['2215', '市值.1']
+        nav = float(df.loc['今日单位净值', '科目名称'])
+        pur_amt_receivable = np.nan
+        sec_clearing_fund = np.nan
+
+        total_df = pd.Series({
+            'fof_id': fof_id,
+            'manager_id': manager_id,
+            'datetime': the_date,
+            'nav': nav,
+            'equity': equity,
+            'total_shares': total_shares,
+            'other_assets': other_assets,
+            'deposit_in_bank': deposit_in_bank,
+            'interest_receivable': interest_receivable,
+            'pur_amt_receivable': pur_amt_receivable,
+            'sec_clearing_fund': sec_clearing_fund,
+            'dividends_receivable': dividends_receivable,
+            'payable_to_managers': payable_to_managers,
+            'payable_to_custodian': payable_to_custodian,
+            'payable_to_admin_service': payable_to_admin_service,
+            'mutual_funds_data': mutual_funds.to_json(orient='records'),
+            'hedge_funds_data': hedge_funds.to_json(orient='records'),
+        })
+        total_df = total_df.to_frame().T
+        now_df = DerivedDataApi().get_hedge_fund_custodian_data(manager_id=manager_id, fof_id_list=[fof_id])
+        if now_df is not None:
+            now_df = now_df.drop(columns=['update_time', 'create_time', 'is_deleted']).astype(total_df.dtypes.to_dict())
+            # merge on all columns
+            df = total_df.round(6).merge(now_df.round(6), how='left', indicator=True, validate='one_to_one')
+            df = df[df._merge == 'left_only'].drop(columns=['_merge'])
+        else:
+            df = total_df
+        if not df.empty:
+            print(df)
+            for the_id in df.id.dropna().unique():
+                DerivedDataApi().delete_hedge_fund_custodian_data(id_to_delete=the_id)
+            df.to_sql(HedgeFundCustodianData.__table__.name, DerivedDatabaseConnector().get_engine(), index=False, if_exists='append')
+
     @staticmethod
     def calc_whole_adjusted_net_value(nav_acc_data: pd.DataFrame) -> pd.DataFrame:
         '''
