@@ -10,6 +10,7 @@ from scipy.stats import gmean
 from functools import partial
 import statsmodels.api as sm
 from statsmodels.tsa.ar_model import AutoReg
+from ..data.api.raw import RawDataApi
 from ..data.api.basic import BasicDataApi
 from ..data.struct import TAAParam, TaaTunerParam, AssetWeight
 from ..data.api.derived import DerivedDataApi
@@ -38,6 +39,7 @@ class SeriesStatUnit:
     recent_3m_ret: float                # 最近3月收益
     recent_6m_ret: float                # 最近6月收益
     y_2020_ret: float                   # 2020年收益
+    last_year_ret: float                # 去年收益
     recent_y1_ret: float                # 近一年收益
     recent_y3_ret: float                # 近三年收益
     recent_y5_ret: float                # 近五年收益
@@ -162,6 +164,7 @@ class Calculator:
             recent_1m_ret=res['recent_1m_ret'],
             recent_4w_ret=res['recent_4w_ret'],
             y_2020_ret=res['y_2020_ret'],
+            last_year_ret=res['last_year_ret'],
             recent_3m_ret=res['recent_3m_ret'],
             recent_6m_ret=res['recent_6m_ret'],
             recent_y1_ret=res['recent_1y_ret'],
@@ -238,6 +241,7 @@ class Calculator:
             recent_1m_ret=res['recent_1m_ret'],
             recent_4w_ret=res['recent_4w_ret'],
             y_2020_ret=res['y_2020_ret'],
+            last_year_ret=res['last_year_ret'],
             recent_3m_ret=res['recent_3m_ret'],
             recent_6m_ret=res['recent_6m_ret'],
             recent_y1_ret=res['recent_1y_ret'],
@@ -993,9 +997,13 @@ class Calculator:
         }
         result = []
         for asset in df:
-            dates = df[asset].index
-            values = df[asset].values
-            dic = Calculator.get_stat_result(dates=dates,values=values).__dict__
+            asset_type = asset_info[asset_info.real_name == asset].asset_type.values[0]
+            dates = df[asset].dropna().index
+            values = df[asset].dropna().values
+            if asset_type != '典型产品':
+                dic = Calculator.get_stat_result(dates=dates,values=values).__dict__
+            else:
+                dic = Calculator.get_stat_result(dates=dates,values=values,frequency='1W').__dict__
             dic['具体指数'] = asset
             dic['类别'] = real_asset_dic[asset]
             result.append(dic)
@@ -1010,6 +1018,93 @@ class Calculator:
         df['卡玛比率'] = df['卡玛比率'].round(2)
         return df
         
+    @staticmethod
+    def get_index_stats(begin_date, end_date, time_para, index_list):
+        try:
+            indicator_dics = {
+                'cumu_ret':'区间收益',
+                'annualized_ret':'年化收益',
+                'annualized_vol':'年化波动',
+                'mdd':'最大回撤',
+                'sharpe':'夏普比率',
+                'ret_over_mdd':'卡玛比率',
+                'downside_std':'下行风险',
+            }
+            _dic = {
+                'pb_mrq':'PB',
+                'pe_ttm':'PE',
+                'roe':'ROE',
+                'ps_ttm':'PS',
+                'dy':'股息率',
+                'eps_ttm':'每股收益',
+                'est_peg':'预测PEG',
+            }
+            result = []
+            begin_date, end_date = RawDataApi().get_date_range(time_para, begin_date, end_date)
+            basic = BasicDataApi()
+            index_info = basic.get_index_info(index_list=index_list)
+            index_name = index_info.set_index('index_id').to_dict()['desc_name']
+            df = basic.get_index_price_dt(index_list=index_list,start_date=begin_date,end_date=end_date).pivot_table(index='datetime',columns='index_id',values='close')
+            result = []
+            for asset in df:
+                dates = df[asset].index
+                values = df[asset].values
+                dic = Calculator.get_stat_result(dates=dates,values=values).__dict__
+                dic['具体指数'] = index_name[asset]
+                dic['index_id'] = asset
+                result.append(dic)
+
+            df = pandas.DataFrame(result).set_index('index_id')
+            df_val = DerivedDataApi().get_index_valuation_last_date(dt=end_date, index_ids=index_list)
+            df_val = df_val.set_index('index_id')
+            df = df[list(indicator_dics.keys())+['具体指数']].rename(columns=indicator_dics)
+            df = df.join(df_val[list(_dic.keys())].rename(columns=_dic))
+            l = ['夏普比率','具体指数','卡玛比率']
+            for col in df:
+                if col not in l and col not in _dic.values():
+                    df[col] = (df[col] * 100).round(2)
+                if col in _dic.values():
+                    df[col] = df[col].round(2)
+            df['夏普比率'] = df['夏普比率'].round(2)
+            df['卡玛比率'] = df['卡玛比率'].round(2)
+            return df.reset_index(drop=True)
+        except Exception as e:
+            print(f'Failed to get data <err_msg> {e} from get_index_stats.__tablename__')
+            return pandas.DataFrame([])
+
+    @staticmethod
+    def index_val_history(begin_date, end_date, time_para, index_id, valuation_type):
+        dic = {
+            'pb_mrq':'PB',
+            'pe_ttm':'PE',
+            'roe':'ROE'
+        }
+        try:
+            derived = DerivedDataApi()
+            index_ids = [index_id]
+            begin_date, end_date = RawDataApi().get_date_range(time_para, begin_date, end_date)
+            index_val = derived.get_index_valuation_develop(index_ids, begin_date, end_date)
+
+            index_val = index_val.rename(columns=dic).set_index('datetime')[[valuation_type]]
+            index_price = BasicDataApi().get_index_price_dt(index_list=index_ids,start_date=begin_date,end_date=end_date)
+
+            index_price = index_price.pivot_table(index='datetime',columns='index_id',values='close')
+            index_price.columns=['index_price']
+            df = index_val.join(index_price).ffill().dropna()
+            _df = Calculator.get_index_pct_data(dates=df.index,
+                                                values=df[valuation_type].values,
+                                                pct_list=[0.3,0.5,0.7],
+                                                windows=df.shape[0],
+                                                min_periods=60)
+            _df.columns = ['30分位','50分位','70分位']
+            df = df.join(_df)
+            #df.loc[:,'中位数'] = df[valuation_type].rolling(window=df.shape[0], min_periods=10).median()
+            #df.loc[:,'百分位'] = df[valuation_type].rolling(window=df.shape[0], min_periods=10).apply(pctrank)
+            return df.dropna()
+        except Exception as e:
+            print(f'failed to get data <err_msg> {e} from derived.index_val_history')
+            return False
+
     @staticmethod
     def get_product_stats(df:pandas.DataFrame, asset_info: pandas.DataFrame):
         df = Calculator.get_asset_stats(df, asset_info)
